﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// このサンプルは、図面上に配置された作図部品から、その属性値を取得し表を作図するものです。
//
// 実行するには、あらかじめサンプル図面"ListUpAttribute.rpcd"を開いてください。
// または、下記の条件を満たす図面を開いてください。
// "PRICE"という名前の属性値（整数を文字にしたもの）を持つ作図部品が"FLOOR"という名前の部分図上にあること。
// 実行は「RootPro CAD」の[アドイン]メニューから[表の作成]を選びます。
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace RCAddInAttributeListCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        String commandName = "表の作成";// コマンド名
        String commandResistName = "DrawListCS";// コマンドマネージャー登録用コマンド名

        /// <summary>
        /// アドイン起動時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AppAddIn_Startup(object sender, EventArgs e)
        {
            // メニューにコマンド追加
            MacroCommandCallback mc = MacroCommand;// 追加するコマンド関数
            CommandManager.AddMacroCommand("RCAddInAttributeListCS", commandName, mc);

            // コマンドマネージャーにコマンド追加 // (コマンドウィンドウで操作するコマンドになります)
            Command command = CommandManager.AddCommand(commandResistName);　// コマンドマネージャー登録用コマンド名
            command.Name = commandName;// コマンド名（実行時にコマンドウィンドウに表示されます）
            // 座標パラメータの追加
            CommandParameterGroupCollection parameterGroups = command.ParameterGroups;
            CommandParameterGroup parameterGroup = parameterGroups.Add(ParameterItemType.Point, 0, "配置点座標(左上)");

            // パラメータの変更イベント登録
            command.ParameterChanged += new CommandParameterChangedEventHandler(command_ParameterChanged); // イベント処理関数名:command_ParameterChanged 
        }

        /// <summary>
        /// アドインの終了時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            // コマンドの削除(終了時に必ず行うこと)
            CommandManager.RemoveMacroCommand(MacroCommand); // 追加したコマンド関数

            // コマンドマネージャーに登録したコマンドに削除(終了時に必ず行うこと)
            CommandManager.RemoveCommand(commandResistName); // 追加したコマンド名
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        /// <summary>
        /// メニューから"表の作成"コマンドを実行したときの処理
        /// </summary>
        private void MacroCommand()
        {
            // コマンド実行(コマンドマネージャー登録用コマンド名を使う)
            CommandManager.ExecuteCommand(commandResistName);
        }

        /// <summary>
        /// パラメータ変更イベント処理 //（配置点座標が入力されたとき）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_ParameterChanged(Object sender, CommandParameterChangedEventArgs e)
        {
            Command command = (Command)sender;
            int id = e.ParameterID;// 変更のあったパラメータID
            ParameterItem parameterItem = command.GetParameterItemByID(id); // パラメータアイテム

            // パラメータアイテムが座標タイプの場合
            if (parameterItem.ParameterType == ParameterItemType.Point)
            {
                PointParameterItem pointParameterItem = (PointParameterItem)parameterItem;
                Point2d point = pointParameterItem.Point; // 配置点座標の取得

                // 表の作成を実行する
                DrawPriceList(point);
            }

            e.CommandState = CommandState.End; // 終了
        }


        /// <summary>
        /// 作図部品の属性を使って、集計表を作成します。
        /// </summary>
        /// <param name="point">表の配置点(左上)</param>
        private void DrawPriceList(Point2d point)
        {
            Document doc = ActiveDocument;// アクティブな図面の取得
            DrawingCollection drawings = doc.Drawings;// 部分図コレクション取得
            Drawing drawing = drawings["FLOOR"];// "FLOOR"部分図上の図形を対象にする

            // FLOOR がなければ終了
            if (drawing == null)
            {
                MessageBox.Show("部分図「FROOR」がありません。サンプル用の図面ファイルを開いてください。");
                return;
            }

            // カーソルを砂時計に
            Cursor.Current = Cursors.WaitCursor;

            // 集計用ハッシュテーブル(キーは作図部品名, 値はint(個数,属性値PRICEの合計)の配列)用意
            Dictionary<string, int[]> priceDict = new Dictionary<string, int[]>(); // Dictionary(System.Collections.Generic名前空間）

            ///// 部分図上の全図形から作図部品を検索する /////
            Shape shape = null;
            shape = drawing.Shapes.GetNextShape(shape);// 部分図にある最初の図形
            while (shape != null)
            {
                // 作図部品の場合
                if (shape is DrawComponentShape)
                {
                    DrawComponentShape drawComponentShape = (DrawComponentShape)shape;

                    // 作図部品定義取得
                    MasterDrawComponent masterDrawComponent = drawComponentShape.Master;

                    // 作図部品定義の名前取得
                    String drawComponentName = masterDrawComponent.Name;

                    // 属性 PRICE がある場合のインデックス取得
                    ShapeAttributeCollection shapeAttributes = drawComponentShape.Attributes;
                    int index = shapeAttributes.IndexOfName("PRICE");
                    if (index >= 0)
                    {
                        // 配置されている作図部品からPRICE値を取得
                        ShapeAttribute shapeAttribute = shapeAttributes[index];// 作図部品定義の属性と同じPRICEのインデックスでPRICE値を取得する
                        String priceText = shapeAttribute.Value;// 金額（文字）
                        int price = int.Parse(priceText);       // 金額文字を数値に変換

                        int[] itemInfo;
                        // 集計用ハッシュテーブルに作図部品定義の名前をキーにしたものが存在しな場合
                        if (!priceDict.TryGetValue(drawComponentName, out itemInfo))
                        {
                            itemInfo = new int[2];
                            itemInfo[0] = 1;        // 個数
                            itemInfo[1] = price;    // 金額

                            // 作図部品の名前をキーに新しい値の追加
                            priceDict.Add(drawComponentName, itemInfo);
                        }
                        // 存在する場合は更新
                        else
                        {
                            itemInfo[0] = itemInfo[0] + 1;        // 個数
                            itemInfo[1] = itemInfo[1] + price;    // 金額

                            priceDict[drawComponentName] = itemInfo;
                        }
                    }
                }

                shape = drawing.Shapes.GetNextShape(shape);// 次の図形
            }

            // 作図部品がなかった場合は終了する
            if (priceDict.Count == 0)
            {
                MessageBox.Show("属性に「PRICE」が設定された作図部品が配置されていません。");
                return;
            }

            // 表の線と文字を作図
            DrawListLineText(point, priceDict);
        }


        /// <summary>
        /// 表の線と文字を作図する
        /// </summary>
        /// <param name="locatePoint">表の配置点(左上)</param>
        /// <param name="priceDict">集計用ハッシュテーブル</param>
        private void DrawListLineText(Point2d locatePoint, Dictionary<string, int[]> priceDict)
        {
            Document doc = ActiveDocument;          // アクティブな図面
            Drawing drawing = doc.CurrentDrawing;   // カレント部分図

            // 文字の既定値を取得
            TextSettings textSettings = doc.TextSettings;

            // 表の行数(項目名＋作図部品の種類の数＋合計）
            int rowCount = 1 + priceDict.Count + 1;

            // 表の列の数
            const int columnCount = 3;

            // 文字の高さ
            double textHeight = textSettings.FontHeight;

            // 列幅
            double[] columnWidth = new double[columnCount];
            // 作図部品名（項目名：部品名）列の幅
            columnWidth[0] = 8 * textHeight; // 1文字の幅が文字高さと同じとして8文字分を列幅にする

            // 個数列の幅
            columnWidth[1] = 3 * textHeight;

            // 金額列の幅
            columnWidth[2] = 5 * textHeight;

            // 表全体の幅
            double listTotalWidth = 0.0;
            for (int i = 0; i < columnCount; i++)
            {
                listTotalWidth += columnWidth[i];
            }

            // セル内のオフセット値
            double offset = textHeight * 0.2; // 1セル内に書く文字の余白

            // セルの高さ
            double cellHight = textHeight + offset * 2;

            // 表全体の高さ
            double listTotalHeight = cellHight * rowCount;

            // UNDO 可能な操作を開始
            doc.UndoManager.BeginUndoUnit();

            ///// 線を描く(表の配置基準点は左上) /////
            // 横線
            for (int i = 0; i <= rowCount; i++)
            {
                // 線
                Point2d point1 = Geometry.CreatePoint(locatePoint.X, locatePoint.Y - cellHight * i); // 始点
                Point2d point2 = Geometry.CreatePoint(locatePoint.X + listTotalWidth, locatePoint.Y - cellHight * i); // 終点
                LineShape line = drawing.Shapes.AddLine(point1, point2); // 線の追加
                line.Linetype = doc.Linetypes.GetLinetypeByNumber(1);// 線種(実線)
            }

            // 縦線
            double lastPointX = 0.0;
            for (int i = 0; i <= columnCount; i++)
            {
                // 線
                double x = 0.0;
                if (i == 0)
                    x = locatePoint.X;
                else
                    x = lastPointX + columnWidth[i - 1];

                Point2d point1 = Geometry.CreatePoint(x, locatePoint.Y); // 始点
                Point2d point2 = Geometry.CreatePoint(x, locatePoint.Y - listTotalHeight); // 終点
                LineShape line = drawing.Shapes.AddLine(point1, point2); // 線の追加
                line.Linetype = doc.Linetypes.GetLinetypeByNumber(1);// 線種(実線)

                lastPointX = x;
            }


            ///// セル内に文字を書く /////
            String[] cellText = new String[3];
            double[] textLocatePointX = new double[3];// 文字配置点X座標1列目、2列目、3列目（1列目は左下、2、3列目は右下を基準）
            textLocatePointX[0] = locatePoint.X + offset;
            textLocatePointX[1] = locatePoint.X + columnWidth[0] + columnWidth[1] - offset;
            textLocatePointX[2] = locatePoint.X + columnWidth[0] + columnWidth[1] + columnWidth[2] - offset;

            // 項目名行
            cellText[0] = "部品名"; // 1列目（部品名）
            cellText[1] = "個数";   // 2列目（個数）
            cellText[2] = "金額"; // 3列目（金額）

            // 配置点Y座標
            double textLocatePointY = locatePoint.Y - cellHight + offset;
            // 書き出し
            DrawPriceText(doc, drawing, cellText, textLocatePointX, textLocatePointY);

            // 部品名、個数、金額（小計）の値を書く
            int totalPartsCount = 0;
            int totalPrice = 0;
            foreach (String key in priceDict.Keys)
            {
                int[] itemInfo = priceDict[key];

                int partsCount = itemInfo[0]; // 部品の数
                int price = itemInfo[1];      // 部品の金額（小計）

                // 合計値更新
                totalPartsCount += partsCount;
                totalPrice += price;

                // 文字列
                cellText[0] = key;// 1列目（部品名）
                cellText[1] = partsCount.ToString();// 2列目（個数）
                cellText[2] = price.ToString();// 3列目（金額）

                // Y座標
                textLocatePointY -= cellHight;

                // 書き出し
                DrawPriceText(doc, drawing, cellText, textLocatePointX, textLocatePointY);

            }

            // 合計行
            // 文字列
            cellText[0] = "合計";// 1列目（部品名）
            cellText[1] = totalPartsCount.ToString();// 2列目（個数）
            cellText[2] = totalPrice.ToString();// 3列目（金額）

            // Y座標
            textLocatePointY -= cellHight;

            // 書き出し
            DrawPriceText(doc, drawing, cellText, textLocatePointX, textLocatePointY);

            // UNDO 可能な操作を終了
            doc.UndoManager.EndUndoUnit();
        }


        /// <summary>
        /// 表の文字を1行分書く
        /// </summary>
        /// <param name="doc"></param>
        /// <param name="drawing"></param>
        /// <param name="textLine">文字列の配列(3)</param>
        /// <param name="textLocatePointX">配置点のX座標</param>
        /// <param name="textLocatePointY">配置点のY座標</param>
        private void DrawPriceText(Document doc, Drawing drawing, String[] textLine, double[] textLocatePointX, double textLocatePointY)
        {
            TextShape textShape;
            Point2d textLocatePoint = Geometry.CreatePoint(0.0, textLocatePointY);

            // 1列目
            textLocatePoint.X = textLocatePointX[0];
            textShape = drawing.Shapes.AddText(textLine[0], textLocatePoint, 0);
            textShape.Color = doc.Colors.GetColorByNumber(1);// 色(黒)
            textShape.Alignment = Alignment.BottomLeft;      // 左下

            // 2列目
            textLocatePoint.X = textLocatePointX[1];
            textShape = drawing.Shapes.AddText(textLine[1], textLocatePoint, 0);
            textShape.Color = doc.Colors.GetColorByNumber(1);// 色(黒)
            textShape.Alignment = Alignment.BottomRight;     // 右下

            // 3列目
            textLocatePoint.X = textLocatePointX[2];
            textShape = drawing.Shapes.AddText(textLine[2], textLocatePoint, 0);
            textShape.Color = doc.Colors.GetColorByNumber(1);// 色(黒)
            textShape.Alignment = Alignment.BottomRight;     // 右下
        }
    }
}
